@@ -1148,7 +1148,7 @@ namespace giac {
     if (has_num_coeff(e))
       return approx_rootof(e,contextptr);
     if (!lop(lvar(e),at_pow).empty()){
-      *logptr(contextptr) << gettext("Algebraic extensions not allowed in a rootof");
+      *logptr(contextptr) << gettext("Algebraic extensions not allowed in a rootof")<<endl;
       return approx_rootof(e,contextptr);
     }
     // should call factor before returning unevaluated rootof
@@ -1164,7 +1164,7 @@ namespace giac {
     if ( (e.type!=_VECT) || (e._VECTptr->size()!=2) )
       return gensizeerr(contextptr);
     if (!lidnt(e).empty())
-      return e;
+      return symbolic(at_rootof,e);
     gen a=e._VECTptr->front(),b=e._VECTptr->back();
     return alg_evalf(a,b,contextptr);
   }
@@ -1471,6 +1471,11 @@ namespace giac {
       return gensizeerr(contextptr);
     expr=v[0];
     var=v[1];
+    // avoid inf recursion like g0(x):=ln(abs(ln(x)));
+    // g1(x,xp):=x/(ln(x))^(xp);g0(g1(x,.3));
+    gen varev=eval(var,1,contextptr); 
+    if (varev!=var && contains(varev,var))
+      return undef;
     if (expr.type==_SYMB){
       unary_function_ptr & u=expr._SYMBptr->sommet;
       if (u==at_exp || u==at_ln || u==at_atan || u==at_abs){
@@ -1800,6 +1805,8 @@ namespace giac {
   }
 
   int sturmsign(const gen & g0,bool strict,GIAC_CONTEXT){
+    int fs=fastsign(g0,contextptr);
+    if (fs) return fs;
     gen g=simplifier(g0,contextptr);
     // first check some operators inv, *, exp, sqrt
     int tmp;
