@@ -312,12 +312,20 @@ namespace giac {
 	  return exponent*dbase*s/v[1];
 	return dexponent*ln(base,contextptr)*s+exponent*dbase*s/v[1];
       }
-      if (vs==3 && s.sommet==at_Beta){
-	gen v0=v[0],v1=v[1],v2=v[2]; 
+      if (vs>=3 && s.sommet==at_Beta){
+	gen v0=v[0],v1=v[1],v2=v[2],v3=v[3]; 
 	if (!is_zero(derive(v0,i,contextptr)) || !is_zero(derive(v1,i,contextptr)) )
 	  return gensizeerr("diff of incomplete beta with respect to non constant 1st or 2nd arg not implemented");
-	// diff/v2(int_0^v2 t^(v0-1)*(1-t)^(v1-1) dt)
+	// diff/v2 of int_0^v2 t^(v0-1)*(1-t)^(v1-1) dt
 	gen tmp=pow(v2,v0-1,contextptr)*pow(1-v2,v1-1,contextptr)*derive(v2,i,contextptr);
+	if (vs==4){
+	  if (is_one(v3))
+	    return tmp/Beta(v0,v1,contextptr);
+	  return gensizeerr(contextptr);
+	  gen v3p=derive(v3,i,contextptr);
+	  if (!is_zero(v3p))
+	    return tmp-pow(v3,v0-1,contextptr)*pow(1-v3,v1-1,contextptr)*v3p;
+	}
 	return tmp;
       }
       if (vs==4 && s.sommet==at_sum){
@@ -364,7 +372,7 @@ namespace giac {
       return symbolic(at_plus,gen(v,_SEQ__VECT));
     }
     // integrate
-    if (s.sommet==at_integrate){
+    if (s.sommet==at_integrate || s.sommet==at_HPINT){
       if (s.feuille.type!=_VECT)
 	return s.feuille;
       vecteur v=*s.feuille._VECTptr;
@@ -508,6 +516,15 @@ namespace giac {
   }
 
   gen derive(const gen & e,const identificateur & i,GIAC_CONTEXT){
+    if (abs_calc_mode(contextptr)==38 && i.id_name[0]>='A' && i.id_name[0]<='Z'){
+      identificateur tmp("xdiff");
+      gen ee=subst(e,i,tmp,true,contextptr);
+      ee=eval(ee,1,contextptr);
+      ee=subst(ee,i,tmp,true,contextptr);
+      ee=derive(ee,tmp,contextptr);
+      ee=subst(ee,tmp,i,true,contextptr);
+      return ee;
+    }
     switch (e.type){
     case _INT_: case _DOUBLE_: case _ZINT: case _CPLX: case _MOD: case _REAL: case _USER: case _FLOAT_:
       return 0;
@@ -988,10 +1005,7 @@ namespace giac {
 	continue;
       }
       if (u==at_tan){
-	if (mode==1)
-	  excluded=mergevecteur(excluded,gen2vecteur(_solve(makesequence(symb_equal(symb_cos(gf),0),x),contextptr)));
-	else
-	  eqs.push_back(symb_cos(gf));
+	excluded=mergevecteur(excluded,gen2vecteur(_solve(makesequence(symb_equal(symb_cos(gf),0),x),contextptr)));
 	continue;
       }
       if (u==at_sin || u==at_cos || u==at_exp || u==at_atan)
@@ -1027,7 +1041,7 @@ namespace giac {
     if (excluded.empty())
       return res.size()==1?res.front():res;
     vecteur tmp;
-    for (int i=0;i<excluded.size();++i){
+    for (int i=0;i<int(excluded.size());++i){
       tmp.push_back(symbolic(at_different,makesequence(x,excluded[i])));
     }
     if (res.size()==1 && res.front()==x)
