@@ -66,6 +66,7 @@ namespace giac {
   const double svg_epaisseur1=200; // thickness=(xmax-xmin)/svg_epaisseur1
 
   struct svg_attribut {
+    double ysurx;
     int color;
     int width;
     int epaisseur_point;
@@ -98,11 +99,20 @@ namespace giac {
 
    string string2mathml(const string & m){
      string s=m;
+#if 0
+     size_t ms=m.size();
+     for (size_t i=0;i<ms;++i){
+       if (m[i]=='\\')
+	 s+="\\\\";
+       else
+	 s+=m[i];
+     }
+#endif
      string t="";
-     string mat[4]={"&","<",">","\n"};
-     string rep[4]={"&amp;","&lt;","&gt;","</mi></mtd></mtr><mtr><mtd><mi>"};
+     string mat[]={"&","<",">","\n"};
+     string rep[]={"&amp;","&lt;","&gt;","</mi></mtd></mtr><mtr><mtd><mi>"};
      //start with & before adding new ones
-     for(int siz=0;siz<4;siz++){
+     for(int siz=0;siz<sizeof(mat)/sizeof(string);siz++){
        int c=0,k=-1,le=s.length();
        while (c<le){
          k=s.find(mat[siz],c);
@@ -482,8 +492,12 @@ namespace giac {
   // --------------------------- provenant de derive ----------------------------
 
   static string mathml_printasderive(const gen & feuille,GIAC_CONTEXT){
-    if (feuille.type!=_VECT || feuille._VECTptr->size()<2)
-      return "<msup><mrow><mo>(</mo>"+gen2mathml(feuille,contextptr)+"<mo>)</mo></mrow><mi>'</mi></msup>";
+    if (feuille.type!=_VECT || feuille._VECTptr->size()<2){
+      if (feuille.type==_IDNT || feuille.type<=_CPLX || feuille.is_symb_of_sommet(at_derive))
+	return gen2mathml(feuille,contextptr)+"'";
+      else
+	return "<msup><mrow><mo>(</mo>"+gen2mathml(feuille,contextptr)+"<mo>)</mo></mrow><mi>'</mi></msup>";
+    }
     vecteur & v = *feuille._VECTptr;
     bool needpar=v[0].type==_SYMB;
     if (v.size()>2)
@@ -769,7 +783,7 @@ namespace giac {
   static string svg_text(gen A, string legende, svg_attribut attr,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
     double x_scale=(xmax-xmin)/10;
     double y_scale=(ymax-ymin)/10;
-    double fontscale=0.4*x_scale;
+    double fontscale=0.3*x_scale;
     double ratio=y_scale/x_scale/0.6;
     if (legende=="")
       return legende;
@@ -778,11 +792,19 @@ namespace giac {
     if (legende[0]=='\"')
       legende=legende.substr(1,end-1);
     gen x=re(A,contextptr); // should take care of legend position
-    gen y(ymax+ymin-im(A,contextptr));
+    if (x.type!=_DOUBLE_){
+      x=evalf(x,1,contextptr);
+      x=evalf_double(x,1,contextptr);
+    }
     if (is_greater(x,xmax,contextptr))
       x=xmax-x_scale;
     if (is_greater(xmin,x,contextptr))
       x=xmin+x_scale;    
+    gen y(ymax+ymin-im(A,contextptr));
+    if (y.type!=_DOUBLE_){
+      y=evalf(y,1,contextptr);
+      y=evalf_double(y,1,contextptr);
+    }
     if (is_greater(y,ymax,contextptr))
       y=ymax-y_scale;
     if (is_greater(ymin,y,contextptr))
@@ -852,10 +874,45 @@ namespace giac {
       + im(A,contextptr).print(contextptr)+"\" x2=\""
       + re(B,contextptr).print(contextptr)+"\" y2=\""
       + im(B,contextptr).print(contextptr)+"\"/>\n";
+    //CERR << s << endl;
     s = s+svg_text(B,legende,attr,xmin,xmax,ymin,ymax,contextptr);
     return s;
   }
 
+  static string svg_vecteur(gen A, gen B, svg_attribut attr, string legende,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
+    string s;
+    // gen thickness((xmax+ymax-xmin-ymin)/svg_epaisseur1);
+    s = svg_segment(A,B,attr,legende,xmin,xmax,ymin,ymax,contextptr);
+    gen Ax,Ay,Bx,By;
+    reim(A,Ax,Ay,contextptr);
+    reim(B,Bx,By,contextptr); 
+    Ax=evalf_double(Ax,1,contextptr); Bx=evalf_double(Bx,1,contextptr);
+    Ay=evalf_double(Ay,1,contextptr); By=evalf_double(By,1,contextptr);
+    if (Ax.type==_DOUBLE_ && Bx.type==_DOUBLE_ && Ay.type==_DOUBLE_ && By.type==_DOUBLE_){
+      double dx=Ax._DOUBLE_val-Bx._DOUBLE_val,dy=Ay._DOUBLE_val-By._DOUBLE_val;
+      double dxy=std::sqrt(dx*dx+dy/attr.ysurx*dy/attr.ysurx);
+      if (dxy){
+	dxy =dxy/(std::min(5.0,dxy/10.0)+std::max(1.0,double(attr.width)));
+	dx = dx/dxy;
+	dy = dy/dxy;
+	dxy = std::min(xmax-xmin,ymax-ymin)/70.0;
+	dx = dx*dxy;
+	dy = dy*dxy;
+	double dxp=-dy/attr.ysurx,dyp=dx*attr.ysurx; // apparent perpendicular
+	dx = dx*std::sqrt(3.0);
+	dy = dy*std::sqrt(3.0);
+	//CERR << "step3 " << dx << " " << dxp << " " << dx+dxp << " " << dy << endl;
+	double dx1=dx+dxp,dx2=dx-dxp,dy1=dy+dyp,dy2=dy-dyp;
+	gen C=B+gen(dx1,dy1);
+	gen D=B+gen(dx2,dy2);
+	//CERR << C << " " << D << endl;
+	s += svg_segment(B,C,attr,"",xmin,xmax,ymin,ymax,contextptr);
+	s += svg_segment(B,D,attr,"",xmin,xmax,ymin,ymax,contextptr);
+      }
+    }
+    return s;
+  }
+
   static string svg_point(gen center, svg_attribut attr, string legende,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
     double svg_width=xmax-xmin;
     double svg_height=ymax-ymin;
@@ -873,8 +930,8 @@ namespace giac {
       return svg_segment(center-dx-i*dy,center+dx-i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center+dx-i*dy,center+dx+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-dx+i*dy,center+dx+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-dx+i*dy,center-dx-i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
     case _POINT_TRIANGLE:
       return svg_segment(center-dx-i*dy,center-dx+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-dx-i*dy,center+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-dx+i*dy,center+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
-    case _POINT_POINT:
-      return svg_segment(center,center,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
+    case _POINT_POINT: // must make a small segment otherwise nothing visible
+      return svg_segment(center-dx/2,center+dx/2,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
     }
     return svg_segment(center-dx-i*dy,center+dx+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-dx+i*dy,center+dx-i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
   }
@@ -986,7 +1043,7 @@ namespace giac {
   }
 
 
-  static string symbolic2svg(const symbolic & mys,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT);
+  static string symbolic2svg(const symbolic & mys,double xmin,double xmax,double ymin,double ymax,double ysurx,GIAC_CONTEXT);
 
   //fonction appelee ssi v est un vecteur
   static string vect2svg(gen v, svg_attribut attr, string name,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
@@ -997,12 +1054,12 @@ namespace giac {
 	return vect2svg(v[0], attr, name,xmin,xmax,ymin,ymax,contextptr);
       }
       if (v[0].type==_SYMB)
-	return symbolic2svg(*v[0]._SYMBptr,xmin,xmax,ymin,ymax,contextptr);
+	return symbolic2svg(*v[0]._SYMBptr,xmin,xmax,ymin,ymax,attr.ysurx,contextptr);
     }
     if (v.subtype==_GROUP__VECT || v._VECTptr->size()>2)
       return svg_polyline(v, attr, name,xmin,xmax,ymin,ymax,contextptr);
     if (v.subtype==_VECTOR__VECT)
-      return svg_segment(v[0],v[1], attr, name,xmin,xmax,ymin,ymax,contextptr);
+      return svg_vecteur(v[0],v[1], attr, name,xmin,xmax,ymin,ymax,contextptr);
     if (v.subtype==_LINE__VECT)
       return svg_line(v[0],v[1], attr, name,xmin,xmax,ymin,ymax,contextptr);
     if (v.subtype==_HALFLINE__VECT)
@@ -1014,7 +1071,7 @@ namespace giac {
 #include <emscripten.h>
 #endif
 
-  static string symbolic2svg(const symbolic & mys,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){ 
+  static string symbolic2svg(const symbolic & mys,double xmin,double xmax,double ymin,double ymax,double ysurx,GIAC_CONTEXT){ 
     int color=default_color(contextptr);
     string name="";
     if (mys.sommet==at_pnt){ 
@@ -1054,7 +1111,7 @@ namespace giac {
 	    return 0;
 	});
 #endif
-      svg_attribut attr={color,width+1,epaisseur_point,type_line,type_point,fill_polygon,hidden_name,ie};
+      svg_attribut attr={ysurx,color,width+1,epaisseur_point,type_line,type_point,fill_polygon,hidden_name,ie};
       if(v.size()==3)
 	name=v[2].print(contextptr);
       if (v[0].type==_VECT){
@@ -1065,11 +1122,18 @@ namespace giac {
       if (v[0].type==_SYMB){ 
 	symbolic figure=*v[0]._SYMBptr; 
 	if (figure.sommet == at_curve){
-	  gen curve=figure.feuille;
-	  return svg_bezier_curve(curve[1],attr,name,xmin,xmax,ymin,ymax,contextptr);
+	  //CERR << attr.ysurx << endl;
+	  gen curve=figure.feuille[1];
+	  string s;
+	  if (type_line>4 && curve.type==_VECT && curve._VECTptr->size()>2){
+	    vecteur & v = *curve._VECTptr;
+	    s=svg_vecteur(v[v.size()/2],v[v.size()/2+1],attr,"",xmin,xmax,ymin,ymax,contextptr);
+	  }
+	  s=s+svg_bezier_curve(curve,attr,name,xmin,xmax,ymin,ymax,contextptr);
+	  return s;
 	}
 	if (figure.sommet == at_pnt)
-	  return symbolic2svg(figure,xmin,xmax,ymin,ymax,contextptr);
+	  return symbolic2svg(figure,xmin,xmax,ymin,ymax,attr.ysurx,contextptr);
 	if (figure.sommet==at_segment || figure.sommet==at_vector){
 	  gen segment=figure.feuille;
 	  return svg_segment(segment[0],segment[1], attr, name,xmin,xmax,ymin,ymax,contextptr); 
@@ -1102,9 +1166,9 @@ namespace giac {
   }
 
 
-  string gen2svg(const gen &e,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
+  string gen2svg(const gen &e,double xmin,double xmax,double ymin,double ymax,double ysurx,GIAC_CONTEXT){
     if (e.type== _SYMB)
-      return symbolic2svg(*e._SYMBptr,xmin,xmax,ymin,ymax,contextptr);
+      return symbolic2svg(*e._SYMBptr,xmin,xmax,ymin,ymax,ysurx,contextptr);
     if (e.type==_VECT){
       string s;
       vecteur v=*e._VECTptr;
@@ -1112,18 +1176,21 @@ namespace giac {
 	if (v[i].type==_SYMB){
 	  symbolic sym=*v[i]._SYMBptr; 
 	  if (sym.sommet==at_pnt)
-	    s=s+symbolic2svg(sym,xmin,xmax,ymin,ymax,contextptr);
+	    s=s+symbolic2svg(sym,xmin,xmax,ymin,ymax,ysurx,contextptr);
 	}
 	if (v[i].type==_VECT){
-	  s=s+gen2svg(v[i],xmin,xmax,ymin,ymax,contextptr);
+	  s=s+gen2svg(v[i],xmin,xmax,ymin,ymax,ysurx,contextptr);
 	}
       }
       return s;
     }
     return "error";
   }
+  string gen2svg(const gen &e,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
+    return gen2svg(e,xmin,xmax,ymin,ymax,1.0,contextptr);
+  }
   string gen2svg(const gen &e,GIAC_CONTEXT){
-    return gen2svg(e,gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,contextptr);
+    return gen2svg(e,gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,1.0,contextptr);
   }
   gen _svg(const gen & g,GIAC_CONTEXT){
     if ( g.type==_STRNG && g.subtype==-1) return  g;
@@ -1147,7 +1214,7 @@ namespace giac {
     if (opstring!="/" && (mys.sommet.ptr()->texprint || mys.sommet==at_different))  
       return mathml_print(mys,contextptr);
     if (mys.sommet==at_pnt) { 
-      svg=svg+symbolic2svg(mys,gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,contextptr);
+      svg=svg+symbolic2svg(mys,gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,1.0,contextptr);
       return "<mtext>"+mys.print(contextptr)+"</mtext>";
     }
     gen tmp,value;
@@ -1314,7 +1381,11 @@ namespace giac {
   }
 #endif
 
-  static string idnt2mathml(const string & sorig){
+  static string indice_mathml(const string & s1,const string & s2){
+    return "<msub><mi>"+s1+"</mi><mn>"+s2+"</mn></msub>";
+  }
+
+  static string idnt2mathml_(const string & sorig){
     string s0=sorig;
     int n=int(s0.size()),j;
     for (j=n-1;j>=2;--j){
@@ -1357,6 +1428,14 @@ namespace giac {
     return s0;
   }
 
+  static string idnt2mathml(const string & s0){
+    int n=int(s0.size()),j;
+    for (j=n-1;j>=1;--j){
+      if (j<n-1 && s0[j]=='_')
+	return indice_mathml(idnt2mathml_(s0.substr(0,j)),s0.substr(j+1,n-1-j));
+    }
+    return "<mi>"+idnt2mathml_(s0)+"</mi>";
+  }
 
   string gen2mathml(const gen &e, string &svg,GIAC_CONTEXT){
     string part_re="", part_im="<mi>i</mi>";
@@ -1406,19 +1485,31 @@ namespace giac {
 	return "<mi>&pi;</mi>";
       if (e==undef)
 	return "<mi>undef</mi>";
-      return  "<mi>"+idnt2mathml(e.print(contextptr))+"</mi>";
+      return  idnt2mathml(e.print(contextptr));
     case _SYMB: {
       gen tmp=aplatir_fois_plus(e);
       if (tmp.type!=_SYMB)
 	return gen2mathml(tmp,svg,contextptr);
       return symbolic2mathml(*tmp._SYMBptr, svg,contextptr);
     }
-    case _VECT:                        
+    case _VECT: {
+      vector<int> V; int p=0;
+      if (is_mod_vecteur(*e._VECTptr,V,p) && p!=0){
+	gen gm=makemodquoted(unmod(e),p);
+	return gen2mathml(gm,svg,contextptr);
+      }
       if (e.subtype==_SPREAD__VECT)
 	return spread2mathml(*e._VECTptr,1,contextptr); //----------------v??rifier le 2??me param??tre
-      if (e.subtype!=_SEQ__VECT && ckmatrix(*e._VECTptr))
+      if (e.subtype!=_SEQ__VECT && ckmatrix(*e._VECTptr)){
+	vector< vector<int> > M; p=0;
+	if (is_mod_matrice(*e._VECTptr,M,p) && p!=0){
+	  gen gm=makemodquoted(unmod(e),p);
+	  return gen2mathml(gm,svg,contextptr);
+	}
 	return matrix2mathml(*e._VECTptr,contextptr);
+      }
       return _VECT2mathml(*e._VECTptr,e.subtype, svg,contextptr);
+    }
     case _SPOL1:
       return _SPOL12mathml(*e._SPOL1ptr,svg,contextptr);
     case _POLY:
